/*

Copyright (c) 2001-2004 Fritz Ganter <ganter@ganter.at>
Website: www.gpsdrive.de

Modifications for ipaq specific version
Copyright (c) 2007 Andrew M. Bishop <amb@gedanken.demon.co.uk>
Website: http://www.geganken.demon.co.uk/gpsdrive-ipaq/

Disclaimer: Please do not use for navigation. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    **********************************************************************/

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "gpsdrive.h"
#include <time.h>
#include <gmodule.h>
#include <gdk/gdktypes.h>
#include <gtk/gtk.h>

/*  Defines for gettext I18n */
# include <libintl.h>
# define _(String) gettext(String)
# ifdef gettext_noop
#  define N_(String) gettext_noop(String)
# else
#  define N_(String) (String)
# endif


extern gchar homedir[500], mapdir[500];
extern gint wpflag, trackflag, muteflag, displaymap_top, displaymap_osm, displaymap_map;
extern gint scaleprefered, milesflag, nauticflag, metricflag;
extern gint debug, scalewanted, savetrack;
extern gchar serialdev[80];
extern gdouble current_long, current_lat;
extern gint needtosave;
extern gchar activewpfile[200];
extern gdouble milesconv;
extern gint printoutsats, minsecmode, nightmode;
extern gint lastnotebook;
extern gint zone;
extern gchar bluecolor[40], trackcolor[40];
extern int serialspeed, disableserial, storetz;
extern int sound_direction, sound_distance, sound_speed, sound_gps;

#define KM2MILES 0.62137119
#define KM2NAUTIC  0.54


/*  write the configurationfile ~/.gpsdrive/gpsdriverc */
void
writeconfig ()
{
  FILE *fp;
  gchar fname[220], str[40];

  g_strlcpy (fname, homedir, sizeof (fname));
  g_strlcat (fname, "gpsdriverc", sizeof (fname));
  fp = fopen (fname, "w");
  if (fp == NULL)
    {
      perror (_("Error saving config file ~/.gpsdrive/gpsdriverc"));
      return;
    }

  fprintf (fp, "showwaypoints = ");
  if (wpflag)
    fprintf (fp, "1\n");
  else
    fprintf (fp, "0\n");

  fprintf (fp, "showtrack = ");
  if (trackflag)
    fprintf (fp, "1\n");
  else
    fprintf (fp, "0\n");

  fprintf (fp, "mutespeechoutput = ");
  if (muteflag)
    fprintf (fp, "1\n");
  else
    fprintf (fp, "0\n");

  fprintf (fp, "showtopomaps = ");
  if (displaymap_top)
    fprintf (fp, "1\n");
  else
    fprintf (fp, "0\n");

  fprintf (fp, "showosmmaps = ");
  if (displaymap_osm)
    fprintf (fp, "1\n");
  else
    fprintf (fp, "0\n");

  fprintf (fp, "showstreetmaps = ");
  if (displaymap_map)
    fprintf (fp, "1\n");
  else
    fprintf (fp, "0\n");

  fprintf (fp, "autobestmap = ");
  if (!scaleprefered)
    fprintf (fp, "1\n");
  else
    fprintf (fp, "0\n");

  fprintf (fp, "units = ");
  if (milesflag)
    fprintf (fp, "miles\n");
  else
    {
      if (metricflag)
	fprintf (fp, "metric\n");
      else
	fprintf (fp, "nautic\n");
    }

  fprintf (fp, "savetrack = ");
  if (savetrack)
    fprintf (fp, "1\n");
  else
    fprintf (fp, "0\n");

  fprintf (fp, "scalewanted = %d\n", scalewanted);

  fprintf (fp, "serialdevice = %s\n", serialdev);

  g_snprintf (str, sizeof (str), "%.6f", current_long);
  g_strdelimit (str, ",", '.');
  fprintf (fp, "lastlong = %s\n", str);

  g_snprintf (str, sizeof (str), "%.6f", current_lat);
  g_strdelimit (str, ",", '.');
  fprintf (fp, "lastlat = %s\n", str);

  fprintf (fp, "waypointfile = ");
  fprintf (fp, "%s\n", activewpfile);

  fprintf (fp, "mapdir = ");
  fprintf (fp, "%s\n", mapdir);

  fprintf (fp, "printoutsats = ");
  if (printoutsats == 0)
    fprintf (fp, "0\n");
  else
    fprintf (fp, "1\n");

  fprintf (fp, "minsecmode = ");
  if (minsecmode == 0)
    fprintf (fp, "0\n");
  else
    fprintf (fp, "1\n");

  fprintf (fp, "nightmode = ");
  fprintf (fp, "%d\n", nightmode);

  fprintf (fp, "lastnotebook = ");
  fprintf (fp, "%d\n", lastnotebook);

  fprintf (fp, "storetz = %d\n", storetz);
  if (storetz)
    fprintf (fp, "timezone = %d\n", zone);
  fprintf (fp, "bigcolor = %s\n", bluecolor);
  fprintf (fp, "trackcolor = %s\n", trackcolor);
  fprintf (fp, "serialspeed = %d\n", serialspeed);
  fprintf (fp, "disableserial = %d\n", disableserial);
  fprintf (fp, "sound_direction = %d\n", sound_direction);
  fprintf (fp, "sound_distance = %d\n", sound_distance);
  fprintf (fp, "sound_speed = %d\n", sound_speed);
  fprintf (fp, "sound_gps = %d\n", sound_gps);


  fclose (fp);
  needtosave = FALSE;
}

/*  read the configurationfile ~/.gpsdrive/gpsdriverc */
void
readconfig ()
{
  FILE *fp;
  gchar fname[220], par1[40], par2[1000], buf[1000];
  gint e;

  g_strlcpy (fname, homedir, sizeof (fname));
  g_strlcat (fname, "gpsdriverc", sizeof (fname));
  fp = fopen (fname, "r");
  if (fp == NULL)
    return;

  if (debug)
    g_print ("reading config file...\n");

  while ((fgets (buf, 1000, fp)) > 0)
    {
      g_strlcpy (par1, "", sizeof (par1));
      g_strlcpy (par2, "", sizeof (par2));
      e = sscanf (buf, "%s = %[^\n]", par1, par2);

      if (debug)
	g_print ("%d [%s] = [%s]\n", e, par1, par2);

      if (e == 2)
	{
	  if ((strcmp (par1, "showwaypoints")) == 0)
	    wpflag = atoi (par2);

	  if ((strcmp (par1, "showtrack")) == 0)
	    trackflag = atoi (par2);

	  if ((strcmp (par1, "mutespeechoutput")) == 0)
	    muteflag = atoi (par2);

	  if ((strcmp (par1, "showtopomaps")) == 0)
	    displaymap_top = atoi (par2);

	  if ((strcmp (par1, "showosmmaps")) == 0)
	    displaymap_osm = atoi (par2);

	  if ((strcmp (par1, "showstreetmaps")) == 0)
	    displaymap_map = atoi (par2);

/*  To set the right sensitive flags bestmap_cb is called later*/
	  if ((strcmp (par1, "autobestmap")) == 0)
	    scaleprefered = !(atoi (par2));


	  if ((strcmp (par1, "units")) == 0)
	    {
	      milesflag = metricflag = nauticflag = FALSE;
	      if ((strcmp (par2, "miles")) == 0)
		{
		  milesflag = TRUE;
		  milesconv = KM2MILES;
		}
	      else
		{
		  if ((strcmp (par2, "metric")) == 0)
		    {
		      metricflag = TRUE;
		      milesconv = 1.0;
		    }
		  else if ((strcmp (par2, "nautic")) == 0)
		    {
		      nauticflag = TRUE;
		      milesconv = KM2NAUTIC;
		    }
		}
	    }

	  if ((strcmp (par1, "savetrack")) == 0)
	    savetrack = atoi (par2);

	  if ((strcmp (par1, "scalewanted")) == 0)
	    scalewanted = atoi (par2);

	  if ((strcmp (par1, "serialdevice")) == 0)
	    g_strlcpy (serialdev, par2, sizeof (serialdev));

	  if ((strcmp (par1, "lastlong")) == 0)
	    current_long = g_strtod (par2, 0);

	  if ((strcmp (par1, "lastlat")) == 0)
	    current_lat = g_strtod (par2, 0);

/*        if ((strcmp (par1, "setdefaultpos")) == 0) */
/*  	setdefaultpos = atoi (par2); */

	  if ((strcmp (par1, "waypointfile")) == 0)
	    g_strlcpy (activewpfile, par2, sizeof (activewpfile));

	  if ((strcmp (par1, "mapdir")) == 0)
	    g_strlcpy (mapdir, par2, sizeof (mapdir));

	  if ((strcmp (par1, "printoutsats")) == 0)
	    printoutsats = atoi (par2);

	  if ((strcmp (par1, "minsecmode")) == 0)
	    minsecmode = atoi (par2);

	  if ((strcmp (par1, "nightmode")) == 0)
	    nightmode = atoi (par2);

	  if ((strcmp (par1, "lastnotebook")) == 0)
	    lastnotebook = atoi (par2);

	  if ((strcmp (par1, "storetz")) == 0)
	    storetz = atoi (par2);
	  if (storetz)
	    if ((strcmp (par1, "timezone")) == 0)
	      zone = atoi (par2);
	  if ((strcmp (par1, "bigcolor")) == 0)
	    g_strlcpy (bluecolor, par2, sizeof (bluecolor));
	  if ((strcmp (par1, "trackcolor")) == 0)
	    g_strlcpy (trackcolor, par2, sizeof (trackcolor));
	  if ((strcmp (par1, "serialspeed")) == 0)
	    serialspeed = atoi (par2);
	  if ((strcmp (par1, "disableserial")) == 0)
	    disableserial = atoi (par2);
	  if ((strcmp (par1, "sound_direction")) == 0)
	    sound_direction = atoi (par2);
	  if ((strcmp (par1, "sound_distance")) == 0)
	    sound_distance = atoi (par2);
	  if ((strcmp (par1, "sound_speed")) == 0)
	    sound_speed = atoi (par2);
	  if ((strcmp (par1, "sound_gps")) == 0)
	    sound_gps = atoi (par2);



	}			/* if e==2 */
    }

  if (debug)
    g_print ("\nreading config file finished\n");
  fclose (fp);
}
